"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const superagent = require("superagent");
const superagentPromise = require("superagent-promise");
const request = superagentPromise(superagent, Promise);
const lruCache = require("lru-cache");
const defaultSettings = {
    maxCachedHandles: 100,
    syncTimeTreshold: 60000,
};
const extendedmind_data_1 = require("./extendedmind-data");
function initializeExtendedMindUtils(apiUrl, settings) {
    let backendApi = apiUrl;
    let config = settings ? settings : {};
    config.apiUrl = apiUrl;
    for (let defaultSetting in defaultSettings) {
        if (defaultSettings.hasOwnProperty(defaultSetting) && !config.hasOwnProperty(defaultSetting)) {
            config[defaultSetting] = defaultSettings[defaultSetting];
        }
    }
    let handleCache;
    let lastHandleCacheReset;
    let headers;
    let lastHeadersReset;
    function fetchPublicItems(handle) {
        return __awaiter(this, void 0, void 0, function* () {
            let publicOwnerUrl = backendApi + "/public/" + handle;
            let backendResponse = yield request.get(publicOwnerUrl).end();
            if (backendResponse.status === 200) {
                return new extendedmind_data_1.ExtendedMindPublicItems(backendResponse.body);
            }
        });
    }
    function refreshPublicItems(handle, cachedItems) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Date.now() - cachedItems.getLastSynced() > config.syncTimeTreshold) {
                let publicOwnerModifiedUrl = backendApi + "/public/" + handle +
                    "?modified=" + cachedItems.getLatestModified();
                let backendResponse = yield request.get(publicOwnerModifiedUrl).end();
                if (backendResponse.status === 200) {
                    cachedItems.updateItems(backendResponse.body);
                }
            }
            return cachedItems;
        });
    }
    function fetchHeaders() {
        return __awaiter(this, void 0, void 0, function* () {
            let publicUrl = backendApi + "/public";
            let backendResponse = yield request.get(publicUrl).end();
            if (backendResponse.status === 200) {
                return new extendedmind_data_1.ExtendedMindHeaders(backendResponse.body);
            }
        });
    }
    function refreshHeaders(cachedHeaders) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Date.now() - cachedHeaders.getLastSynced() > config.syncTimeTreshold) {
                let publicOwnerModifiedUrl = backendApi + "/public" +
                    "?modified=" + cachedHeaders.getLatestModified();
                let backendResponse = yield request.get(publicOwnerModifiedUrl).end();
                if (backendResponse.status === 200) {
                    cachedHeaders.updateHeaders(backendResponse.body);
                }
            }
            return cachedHeaders;
        });
    }
    function getPublicItems(handle) {
        return __awaiter(this, void 0, void 0, function* () {
            let todayMidnight = new Date().setUTCHours(0, 0, 1, 0);
            if (!lastHandleCacheReset || lastHandleCacheReset < todayMidnight) {
                lastHandleCacheReset = Date.now();
                handleCache = lruCache({ max: config.maxCachedHandles });
            }
            const cachedItems = handleCache.get(handle);
            const items = cachedItems ? yield refreshPublicItems(handle, cachedItems) : yield fetchPublicItems(handle);
            handleCache.set(handle, items);
            return items;
        });
    }
    function getHeaders() {
        return __awaiter(this, void 0, void 0, function* () {
            let todayMidnight = new Date().setUTCHours(0, 0, 1, 0);
            if (!lastHeadersReset || lastHeadersReset < todayMidnight) {
                lastHeadersReset = Date.now();
                headers = undefined;
            }
            headers = headers ? yield refreshHeaders(headers) : yield fetchHeaders();
            return headers;
        });
    }
    function getPreviewItem(ownerUUID, itemUUID, previewCode) {
        return __awaiter(this, void 0, void 0, function* () {
            let backendResponse = yield request.get(backendApi + "/" + ownerUUID + "/item/" +
                itemUUID + "/preview/" + previewCode).end();
            if (backendResponse.status === 200) {
                return backendResponse.body;
            }
        });
    }
    function getInfo(latest, history) {
        return __awaiter(this, void 0, void 0, function* () {
            let url = backendApi + "/info";
            if (latest === true) {
                url += "?latest=true";
                if (history === true) {
                    url += "&history=true";
                }
            }
            let infoResponse = yield request.get(url)
                .set("Accept", "application/json")
                .end();
            return infoResponse.body;
        });
    }
    return {
        getHeaders: function () {
            return __awaiter(this, void 0, void 0, function* () {
                return yield getHeaders();
            });
        },
        getInfo: function (latest, history) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield getInfo(latest, history);
            });
        },
        getPreviewItem: function (ownerUUID, itemUUID, previewCode) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield getPreviewItem(ownerUUID, itemUUID, previewCode);
            });
        },
        getPublicItems: function (handle) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield getPublicItems(handle);
            });
        },
    };
}
exports.initializeExtendedMindUtils = initializeExtendedMindUtils;
;
//# sourceMappingURL=extendedmind.js.map